==> org-lsst-ccs-image-utilities-3.1.0-sources.jar: org/lsst/ccs/utilities/ccd/CornerRaft.java <==
package org.lsst.ccs.utilities.ccd;

import org.lsst.ccs.utilities.ccd.Reb.RebType;

/**
 * A Corner Raft Geometry object.
 * In accordance to LCA-13381 a Corner Raft geometry is composed of three 
 * sensors: two guider and one wavefront sensor.
 * 
 * @author The LSST CCS Team
 */
public class CornerRaft extends Raft {

    public static enum CornerRaftPosition {
        CR04(0,4), CR00(0,0), CR40(4,0), CR44(4,4);
        
        private final int parallelPosition;
        private final int serialPosition;
        
        CornerRaftPosition(int parallelPosition, int serialPosition) {
            this.parallelPosition = parallelPosition;
            this.serialPosition = serialPosition;
        }
        
        public int getParallelPosition() {
            return parallelPosition;
        }
        
        public int getSerialPosition() {
            return serialPosition;
        }
    }
    
    private final CornerRaftPosition position;
    
    /**
     * Create a Raft object for a given type of CCD.The resulting Raft is empty, having no Rebs installed.
     * 
     * @param type The type of CCD in this Raft.
     * @param position
     */
    public CornerRaft(CCDType type, CornerRaftPosition position) {
        this(RaftGeometryConstants.createRaftGeometryConstants(type), type, position);
    }
    
    CornerRaft(RaftGeometryConstants raftConstants, CCDType type, CornerRaftPosition position) {        
        super(raftConstants,type);        
        this.position = position;
    }

    @Override
    protected void populateRebList() {
        rebList.add(new Reb(RebType.WAVEFRONT, "RebW", this));
        rebList.add(new Reb(RebType.GUIDER, "RebG", this));
    }
    
    @Override
    protected void addCCDtoReb(CCD child, int p, int s) {
        if (child instanceof WavefrontCCD ) {
            rebList.get(0).addCCD(child);
        } else if ( child instanceof GuiderCCD ) {
            rebList.get(1).addCCD(child);
        } else {
            throw new RuntimeException("Only Guider or Wavefront sensor can be added to a CornerRaft. Type "+child.getType()+" not allowed.");
        }
    }
    
    public CornerRaftPosition getCornerRaftPosition() {
        return position;
    }
        
    
    
    /**
     * Utility static method to create a fully Reb/CCD populated CornerRaft for the
     * provided position.
     * 
     * @param position  The position of the CornerRaft.
     * @return      A fully populated Corner Raft.
     */
    public static CornerRaft createCornerRaft(CornerRaftPosition position) {
        CCDType guiderType = new ITLCCDType();
        CornerRaft raft = new CornerRaft(guiderType, position);
        GuiderCCD guider0 = GuiderCCD.createGuiderCCD(guiderType, GuiderCCD.GuiderType.GUIDER0);
        GuiderCCD guider1 = GuiderCCD.createGuiderCCD(guiderType, GuiderCCD.GuiderType.GUIDER1);
        
        WFCCDType wftype = new WFCCDType();        
        WavefrontCCD wflow = WavefrontCCD.createWavefrontCCD(wftype, WavefrontCCD.WavefrontType.WF0);
        WavefrontCCD wfhigh = WavefrontCCD.createWavefrontCCD(wftype, WavefrontCCD.WavefrontType.WF1);

        //This is a hack because we don't add WavefrontCCDs by parallel and serial index.
        raft.addCCDtoReb(wflow, 0, 0);
        raft.addCCDtoReb(wfhigh, 0, 1);
        wflow.setParallelPosition(0);
        wflow.setSerialPosition(0);
        wfhigh.setParallelPosition(0);
        wfhigh.setSerialPosition(1);
        int x,y;
        switch(position) {
            case CR44:
                x = getXcoord(wflow, raft,0,0);
                y = getYcoord(wflow, raft,0,0);
                raft.addGeometry(wflow, x+wflow.getWidth(), y);
                raft.addGeometry(wfhigh, x, y);                
                raft.addChildGeometry(guider0, 1, 0);
                raft.addChildGeometry(guider1, 0, 1);
                break;
            case CR04:
                x = getXcoord(wflow, raft,2,0);
                y = getYcoord(wflow, raft,2,0);
                raft.addGeometry(wflow, x+wflow.getWidth(), y);
                raft.addGeometry(wfhigh, x, y);
                raft.addChildGeometry(guider0, 1, 0);
                raft.addChildGeometry(guider1, 0, 1);
                break;
            case CR00:
                x = getXcoord(wflow, raft,2,2);
                y = getYcoord(wflow, raft,2,2);
                raft.addGeometry(wflow, x+wflow.getWidth(), y);
                raft.addGeometry(wfhigh, x, y);
                raft.addChildGeometry(guider0, 1, 0);
                raft.addChildGeometry(guider1, 0, 1);
                break;
            case CR40:
                x = getXcoord(wflow, raft,0,2);
                y = getYcoord(wflow, raft,0,2);
                raft.addGeometry(wflow, x+wflow.getWidth(), y);
                raft.addGeometry(wfhigh, x, y);
                raft.addChildGeometry(guider0, 1, 0);
                raft.addChildGeometry(guider1, 0, 1);
                break;
        }
               
        return raft;
    }
    
    private static int getXcoord(WavefrontCCD child, CornerRaft raft, int p, int s) {
        return (2-p) * (2*child.getWidth()) + (2-p)*raft.getRaftGeometryConstants().getGap_inx() + (int)raft.getRaftGeometryConstants().getGap_outx();
    }
    private static int getYcoord(WavefrontCCD child, CornerRaft raft, int p, int s) {
        return (raft.getSerialChildrenCount() - 1 - s) * (child.getHeight() + raft.getRaftGeometryConstants().getGap_iny()) +(int)raft.getRaftGeometryConstants().getGap_outy();        
    }
    
    public static int getXoff1() {
        return 249;
    }
    public static int getXoff2() {
        return 138;
    }    
    public static int getYoff1() {
        return 151;
    }
    public static int getYoff2() {
        return 236;
    }
    public static int getOverhang() {
        return 100;
    }
    public static int getCrdim() {
        return 11630;
    }
}

==> org-lsst-ccs-image-utilities-3.1.14-sources.jar: org/lsst/ccs/utilities/ccd/CornerRaft.java <==
package org.lsst.ccs.utilities.ccd;

import org.lsst.ccs.utilities.ccd.Reb.RebType;

/**
 * A Corner Raft Geometry object.
 * In accordance to LCA-13381 a Corner Raft geometry is composed of three 
 * sensors: two guider and one wavefront sensor.
 * 
 * @author The LSST CCS Team
 */
public class CornerRaft extends Raft {

    public static enum CornerRaftPosition {
        CR04(0,4), CR00(0,0), CR40(4,0), CR44(4,4);
        
        private final int parallelPosition;
        private final int serialPosition;
        
        CornerRaftPosition(int parallelPosition, int serialPosition) {
            this.parallelPosition = parallelPosition;
            this.serialPosition = serialPosition;
        }
        
        public int getParallelPosition() {
            return parallelPosition;
        }
        
        public int getSerialPosition() {
            return serialPosition;
        }
    }
    
    private final CornerRaftPosition position;
    
    /**
     * Create a Raft object for a given type of CCD.The resulting Raft is empty, having no Rebs installed.
     * 
     * @param type The type of CCD in this Raft.
     * @param position
     */
    public CornerRaft(CCDType type, CornerRaftPosition position) {
        this(RaftGeometryConstants.createRaftGeometryConstants(type), type, position);
    }
    
    CornerRaft(RaftGeometryConstants raftConstants, CCDType type, CornerRaftPosition position) {        
        super(raftConstants,type);        
        this.position = position;
    }

    @Override
    protected void populateRebList() {
        rebList.add(new Reb(RebType.WAVEFRONT, "RebW", this));
        rebList.add(new Reb(RebType.GUIDER, "RebG", this));
    }
    
    @Override
    protected void addCCDtoReb(CCD child, int p, int s) {
        if (child instanceof WavefrontCCD ) {
            rebList.get(0).addCCD(child);
        } else if ( child instanceof GuiderCCD ) {
            rebList.get(1).addCCD(child);
        } else if (rebList.get(0).isAuxtelREB()){
            rebList.get(0).addCCD(child);
        } else {
            throw new RuntimeException("Only Guider or Wavefront sensor can be added to a CornerRaft. Type "+child.getType()+" not allowed.");
        }
    }
    
    public CornerRaftPosition getCornerRaftPosition() {
        return position;
    }
          
    /**
     * Utility static method to create a fully Reb/CCD populated CornerRaft for the
     * provided position.
     * 
     * @param position  The position of the CornerRaft.
     * @return      A fully populated Corner Raft.
     */
    public static CornerRaft createCornerRaft(CornerRaftPosition position) {
        CCDType guiderType = new ITLCCDType();
        CornerRaft raft = new CornerRaft(guiderType, position);
        GuiderCCD guider0 = GuiderCCD.createGuiderCCD(guiderType, GuiderCCD.GuiderType.GUIDER0);
        GuiderCCD guider1 = GuiderCCD.createGuiderCCD(guiderType, GuiderCCD.GuiderType.GUIDER1);
        
        WFCCDType wftype = new WFCCDType();        
        WavefrontCCD wflow = WavefrontCCD.createWavefrontCCD(wftype, WavefrontCCD.WavefrontType.WF0);
        WavefrontCCD wfhigh = WavefrontCCD.createWavefrontCCD(wftype, WavefrontCCD.WavefrontType.WF1);

        //This is a hack because we don't add WavefrontCCDs by parallel and serial index.
        raft.addCCDtoReb(wflow, 0, 0);
        raft.addCCDtoReb(wfhigh, 0, 1);
        wflow.setParallelPosition(0);
        wflow.setSerialPosition(0);
        wfhigh.setParallelPosition(0);
        wfhigh.setSerialPosition(1);
        int x,y;
        switch(position) {
            case CR44:
                x = getXcoord(wflow, raft,0,0);
                y = getYcoord(wflow, raft,0,0);
                raft.addGeometry(wflow, x+wflow.getWidth(), y);
                raft.addGeometry(wfhigh, x, y);                
                raft.addChildGeometry(guider0, 1, 0);
                raft.addChildGeometry(guider1, 0, 1);
                break;
            case CR04:
                x = getXcoord(wflow, raft,2,0);
                y = getYcoord(wflow, raft,2,0);
                raft.addGeometry(wflow, x+wflow.getWidth(), y);
                raft.addGeometry(wfhigh, x, y);
                raft.addChildGeometry(guider0, 1, 0);
                raft.addChildGeometry(guider1, 0, 1);
                break;
            case CR00:
                x = getXcoord(wflow, raft,2,2);
                y = getYcoord(wflow, raft,2,2);
                raft.addGeometry(wflow, x+wflow.getWidth(), y);
                raft.addGeometry(wfhigh, x, y);
                raft.addChildGeometry(guider0, 1, 0);
                raft.addChildGeometry(guider1, 0, 1);
                break;
            case CR40:
                x = getXcoord(wflow, raft,0,2);
                y = getYcoord(wflow, raft,0,2);
                raft.addGeometry(wflow, x+wflow.getWidth(), y);
                raft.addGeometry(wfhigh, x, y);
                raft.addChildGeometry(guider0, 1, 0);
                raft.addChildGeometry(guider1, 0, 1);
                break;
        }
               
        return raft;
    }
    
    public static CornerRaft createAuxtelRaft() {
    
        CCDType guiderType = new ITLCCDType();
        CornerRaft raft = new CornerRaft(guiderType, CornerRaftPosition.CR00);
        raft.getRebs().remove(1); // Get rid of the RebG
        raft.getRebs().get(0).setIsAuxtelREB(true);
        
        CCDType scienceType = new ITLCCDType();
        CCD scienceCCD = CCD.createCCD(scienceType);
        
        raft.addChildGeometry(scienceCCD, 0, 0);
        return raft;
    }
    
    private static int getXcoord(WavefrontCCD child, CornerRaft raft, int p, int s) {
        return (2-p) * (2*child.getWidth()) + (2-p)*raft.getRaftGeometryConstants().getGap_inx() + (int)raft.getRaftGeometryConstants().getGap_outx();
    }
    private static int getYcoord(WavefrontCCD child, CornerRaft raft, int p, int s) {
        return (raft.getSerialChildrenCount() - 1 - s) * (child.getHeight() + raft.getRaftGeometryConstants().getGap_iny()) +(int)raft.getRaftGeometryConstants().getGap_outy();        
    }
    
    public static int getXoff1() {
        return 249;
    }
    public static int getXoff2() {
        return 138;
    }    
    public static int getYoff1() {
        return 151;
    }
    public static int getYoff2() {
        return 236;
    }
    public static int getOverhang() {
        return 100;
    }
    public static int getCrdim() {
        return 11630;
    }
}
